use bevy::{
    prelude::{default, Bundle, Component, Name, Vec2},
    reflect::Reflect,
};
use bevy_prototype_lyon::{
    entity::ShapeBundle,
    prelude::{Fill, GeometryBuilder, PathBuilder, Stroke},
    shapes::Line,
};

use crate::{
    datatypes::angle::Angle,
    turtle::{Colors, Precision, TurtleCommand, TurtleCommands},
};

use super::turtle_shapes;

#[derive(Bundle, Reflect, Default)]
pub struct TurtleDrawLine {
    #[reflect(ignore)]
    line: ShapeBundle,
    name: Name,
    marker: LineMarker,
}

#[derive(Component, Default, Reflect)]
struct LineMarker;

impl TurtleDrawLine {
    pub(crate) fn new(start: Vec2, _end: Vec2, index: u64) -> Self {
        let bundle = ShapeBundle {
            path: GeometryBuilder::build_as(&Line(start, start)),
            ..default()
        };
        Self {
            line: bundle,
            name: Name::new(format!("Line {}", index)),
            marker: LineMarker,
        }
    }
}

#[derive(Bundle, Reflect, Default)]

pub struct TurtleDrawCircle {
    #[reflect(ignore)]
    line: ShapeBundle,
    name: Name,
    marker: CircleMarker,
}

#[derive(Component, Default, Reflect)]
struct CircleMarker;

impl TurtleDrawCircle {
    pub(crate) fn new(
        center: Vec2,
        radii: Vec2,
        angle: Angle<Precision>,
        index: u64,
        start: Vec2,
        end: Vec2,
    ) -> Self {
        let mut path_builder = PathBuilder::new();
        path_builder.move_to(start);
        // The center point of the radius - this is responsible for the orientation of the ellipse,
        // then the radii in x and y direction - this can be rotated using the x_rotation parameter,
        // then the angle - the part of the circle that will be drawn like (PI/2.0) for a quarter circle,
        // then the x_rotation (maybe the rotation of the radii?)
        path_builder.arc(center, radii, angle.to_radians().value(), 0.);
        let line = path_builder.build();
        println!("Draw Circle: {} {} {:?}", center, radii, angle);

        let bundle = ShapeBundle {
            path: GeometryBuilder::build_as(&line),
            ..default()
        };
        Self {
            line: bundle,
            name: Name::new(format!("Circle {}", index)),
            marker: CircleMarker,
        }
    }
}

#[derive(Bundle)]
pub struct Turtle {
    colors: Colors,
    commands: TurtleCommands,
    name: Name,
    shape: ShapeBundle,
}

impl Default for Turtle {
    fn default() -> Self {
        let bundle = ShapeBundle {
            path: GeometryBuilder::build_as(&turtle_shapes::turtle()),
            ..default()
        };
        Self {
            colors: Colors::default(),
            commands: TurtleCommands::new(vec![]),
            name: Name::new("Turtle"),
            shape: bundle,
        }
    }
}

impl Turtle {
    /*     pub fn set_color(&mut self, color: Color) {
        self.colors.color = color;
    }
    pub fn set_fill_color(&mut self, color: Color) {
        self.colors.fill_color = color;
    }
    pub fn get_colors(&self) -> &Colors {
        &self.colors
    }
    pub fn forward(&mut self) -> &mut Self {
        self.commands
            .commands
            .push(TurtleCommand::Forward(Length(100.0)));
        self
    } */
    pub fn set_commands(&mut self, commands: Vec<TurtleCommand>) {
        self.commands = TurtleCommands::new(commands);
    }
}
